// Diafaan SMS Server Basic File import and export script
// AddReference System.dll
//
using System;
using System.Collections;
using System.Timers;
using System.IO;

namespace DiafaanMessageServer
{
	public class ConnectorScript : IScript
	{
		private IScriptHost host = null;
		private int timerInterval = 5000;
		private string fileOutFolder = @"C:\temp\out\";
		private string fileOutFilter = "*.txt";
		private string fileInFolder = @"C:\temp\in\";
		private string fileInExtension = "txt";
		private string defaultPhoneNumber = "+1###########";
		private System.Timers.Timer filePollTimer;
  
		public void OnLoad(IScriptHost host)
		{
			try {
				this.host = host;
				filePollTimer = new System.Timers.Timer(timerInterval);
				filePollTimer.Elapsed += new ElapsedEventHandler(OnTimedEvent);
				filePollTimer.Enabled = true;
			}
			catch (Exception ex) {
				PostEventLog(ex.Message, ex.ToString(), EventLog.Error);
			}
		}

		public void OnUnload()
		{
			try {
				filePollTimer.Enabled = false;
				filePollTimer = null;
			}
			catch (Exception ex) {
				PostEventLog(ex.Message, ex.ToString(), EventLog.Error);
			}
		}
		private void OnTimedEvent(Object source, ElapsedEventArgs e)
		{
			string phoneNumber = defaultPhoneNumber;
			
			try {
				foreach (string fileName in Directory.GetFiles(fileOutFolder, fileOutFilter)) {
					try {
						string messageText = File.ReadAllText(fileName);
						if (messageText.Contains("\r")) {
							phoneNumber = messageText.Substring(0, messageText.IndexOf('\r')).Trim();
							messageText = messageText.Substring(messageText.IndexOf('\r')).Trim();
						}
						PostSendMessage(phoneNumber, "", messageText, "", "", "", "", 0);
						File.Delete(fileName);
					}
					catch (Exception ex2) {
						PostEventLog(ex2.Message, ex2.ToString(), EventLog.Error);
					}
				}
			}
			catch (Exception ex) {
				PostEventLog(ex.Message, ex.ToString(), EventLog.Error);
			}
		}
		private void OnMessageReceived(string fromAddress, string toAddress, string message, string messageType,
		                               string messageId, string smsc, string pdu, string gateway,
		                               DateTime sendTime, DateTime receiveTime)
		{
			int counter = 1;
			try {
				if (!string.IsNullOrEmpty(fileInFolder)) {
					string uniqueFileName = DateTime.Now.ToString("yyyyMMddHHmmss");
					string fileName = Path.Combine(fileInFolder, string.Format(@"{0}.{1}", uniqueFileName, fileInExtension));
					while (File.Exists(fileName)) {
						fileName = Path.Combine(fileInFolder, string.Format(@"{0}.{1}", uniqueFileName + "-" + (counter++).ToString(), fileInExtension));
					}
					File.WriteAllText(fileName , fromAddress + Environment.NewLine + message);
				}
			}
			catch (Exception ex) {
				PostEventLog(ex.Message, ex.ToString(), EventLog.Error);
			}
		}
		private void OnSendMessageResult(string toAddress, string fromAddress, string message, string messageType,
		                               	 string messageId, string pdu, string gateway, int statusCode,
		                               	 string statusText, string errorCode, string errorText,
		                               	 DateTime sendTime, DateTime receiveTime, string userId, string userInfo)
		{
			//
			// DO NOTHING
			//
		}
		private void OnSendMessageResultUpdate(string messageId, int statusCode, string statusText, 
		                                       string errorCode, string errorText, DateTime receiveTime)
		{
			//
			// DO NOTHING
			//
		}
		#region Connector Script helper functions
		private enum StatusCode {Sent=200, Received=201, SendError=300}
		private enum EventLog {Information=0, Warning=1, Error=2}
		private enum TraceLog {Information=0, Send=1, Receive=2}

		public void OnMessagePacket(Hashtable messagePacket)
		{
			if (GetPacketString(messagePacket, "PacketName", "") == "MessageIn") {
				OnMessageReceived(GetPacketString(messagePacket, "From", ""),
				                	GetPacketString(messagePacket, "To", ""),
				                	GetPacketString(messagePacket, "Message", ""),
				                	GetPacketString(messagePacket, "MessageType", ""),
				                	GetPacketString(messagePacket, "MessageId", ""),
				                	GetPacketString(messagePacket, "SMSC", ""),
				                	GetPacketString(messagePacket, "PDU", ""),
				                	GetPacketString(messagePacket, "Gateway", ""),
				                	GetPacketDateTime(messagePacket, "SendTime", DateTime.Now),
				                	GetPacketDateTime(messagePacket, "ReceiveTime", DateTime.Now));
			}
			else if (GetPacketString(messagePacket, "PacketName", "") == "MessageOutResult") {
				OnSendMessageResult(GetPacketString(messagePacket, "To", ""),
				                    GetPacketString(messagePacket, "From", ""),
					                	GetPacketString(messagePacket, "Message", ""),
					                	GetPacketString(messagePacket, "MessageType", ""),
					                	GetPacketString(messagePacket, "MessageId", ""),
					                	GetPacketString(messagePacket, "PDU", ""),
					                	GetPacketString(messagePacket, "Gateway", ""),
					                	GetPacketInteger(messagePacket, "StatusCode", -1),
					                	GetPacketString(messagePacket, "StatusText", ""),
					                	GetPacketString(messagePacket, "ErrorCode", ""),
					                	GetPacketString(messagePacket, "ErrorText", ""),
					                	GetPacketDateTime(messagePacket, "SendTime", DateTime.Now),
					                	GetPacketDateTime(messagePacket, "ReceiveTime", DateTime.MaxValue),
					                	GetPacketString(messagePacket, "UserId", ""),
					                	GetPacketString(messagePacket, "UserInfo", ""));
			}
			else if (GetPacketString(messagePacket, "PacketName", "") == "MessageOutResultUpdate") {
				OnSendMessageResultUpdate(GetPacketString(messagePacket, "MessageId", ""),
								                	GetPacketInteger(messagePacket, "StatusCode", -1),
								                	GetPacketString(messagePacket, "StatusText", ""),
					  			              	GetPacketString(messagePacket, "ErrorCode", ""),
					        			        	GetPacketString(messagePacket, "ErrorText", ""),
					        			        	GetPacketDateTime(messagePacket, "ReceiveTime", DateTime.MaxValue));
			}
		}
		private void PostSendMessage(string toAddress, string fromAddress, string messageText,
		                             string messageType, string gateway, string userId, string userInfo, int priority)
		{
			Hashtable messagePacket = new Hashtable();
			
			messagePacket.Add("PacketName", "MessageOut");
			messagePacket.Add("To", toAddress);
			messagePacket.Add("From", fromAddress);
			messagePacket.Add("Message", messageText);
			messagePacket.Add("MessageType", messageType);
			messagePacket.Add("Gateway", gateway);
			messagePacket.Add("UserId", userId);
			messagePacket.Add("UserInfo", userInfo);
			messagePacket.Add("Priority", priority.ToString());
			host.PostMessagePacket(messagePacket);			
		}
		private void PostEventLog(string eventMessage, string eventDescription, EventLog eventType)
		{
			Hashtable messagePacket = new Hashtable();
			
			messagePacket.Add("PacketName", "EventLog");
			messagePacket.Add("EventType", ((int) eventType).ToString());
			messagePacket.Add("EventMessage", eventMessage);
			messagePacket.Add("EventDescription", eventDescription);
			host.PostMessagePacket(messagePacket);			
		}
		private void PostTraceLog(string traceLogMessage, TraceLog logType)
		{
			Hashtable messagePacket = new Hashtable();
			
			messagePacket.Add("PacketName", "TraceLog");
			messagePacket.Add("TraceLogType", ((int) logType).ToString());
			messagePacket.Add("TraceLogMessage", traceLogMessage);
			host.PostMessagePacket(messagePacket);			
		}
		private string GetPacketString(Hashtable hashtable, string key, string defaultValue)
		{
			try {
				if (!hashtable.ContainsKey(key) || (hashtable[key] == null) || (hashtable[key].GetType() != typeof(string))) return defaultValue;
				return (string) hashtable[key];
			}
			catch {}
			return defaultValue;
		}
		private int GetPacketInteger(Hashtable hashtable, string key, int defaultValue)
		{
			try {
				if (!hashtable.ContainsKey(key) || (hashtable[key] == null) || (hashtable[key].GetType() != typeof(string))) return defaultValue;
				return Convert.ToInt32((string) hashtable[key]);
			}
			catch {}
			return defaultValue;
		}
		private DateTime GetPacketDateTime(Hashtable hashtable, string key, DateTime defaultValue)
		{
			try {
				if (!hashtable.ContainsKey(key) || (hashtable[key] == null) || (hashtable[key].GetType() != typeof(string))) return defaultValue;
				return new DateTime(Convert.ToInt64((string) hashtable[key]));
			}
			catch {}
			return defaultValue;
		}
		#endregion
	}
}
