// Diafaan SMS Server Scripting Gateway skeleton script
// AddReference System.dll
// AddReference System.Web.dll
//
using System;
using System.Collections;

namespace DiafaanMessageServer
{
	public class GatewayScript : IScript
	{
		private IScriptHost host = null;
  
		public void OnLoad(IScriptHost host)
		{
			this.host = host;
			//
			// TODO: Add initialization code.
			//
		}

		public void OnUnload()
		{
			//
			// TODO: Add cleanup code, make sure to remove (Timer) event handlers here
			//
		}
		private void OnMessageToSend(string recordId, string toAddress, string fromAddress, 
		                             string message, string messageType)
		{
			String URI = "http://www.hostname.com/page.html";

			try {
				// Add recipient number to the URI
				URI += "?to=" + System.Web.HttpUtility.UrlEncode(toAddress);
				// Add message text to the URI
				URI += "&message=" + System.Web.HttpUtility.UrlEncode(message);
				// TODO: Add other fields to the URI
				
				// Send the web request URI to the web server with a GET request
				System.Net.WebRequest req = System.Net.WebRequest.Create(URI);
				System.Net.WebResponse resp = req.GetResponse();
				System.IO.StreamReader sr = new System.IO.StreamReader(resp.GetResponseStream());
				string response = sr.ReadToEnd().Trim();			//
				PostSendResult(recordId, "", StatusCode.Sent, "", "", "", false);
			}
			catch (Exception e) {
				PostSendResult(recordId, "", StatusCode.SendError, "Error: " + e.Message, "", "", false);
			}
		}
		private void OnStatusRequest(string messageId)
		{
			// TODO: Send status request query
		}
		#region Gateway Script helper functions
		private enum StatusCode {Sent=200, Received=201, SendError=300}
		private enum EventLog {Information=0, Warning=1, Error=2}
		private enum TraceLog {Information=0, Send=1, Receive=2}

		public void OnMessagePacket(Hashtable messagePacket)
		{
			if (GetPacketString(messagePacket, "PacketName", "") == "MessageOut") {
				OnMessageToSend(GetPacketString(messagePacket, "RecordId", ""),
				                GetPacketString(messagePacket, "To", ""),
				                GetPacketString(messagePacket, "From", ""),
				                GetPacketString(messagePacket, "Message", ""),
				                GetPacketString(messagePacket, "MessageType", ""));
			}
			else if (GetPacketString(messagePacket, "PacketName", "") == "StatusRequest") {
				OnStatusRequest(GetPacketString(messagePacket, "MessageId", ""));
			}
		}
		private void PostReceiveMessage(string toAddress, string fromAddress, string messageText,
		                                string messageType, string messageId, DateTime sendTime,
		                                DateTime receiveTime)
		{
			Hashtable messagePacket = new Hashtable();
			
			messagePacket.Add("PacketName", "MessageIn");
			messagePacket.Add("To", toAddress);
			messagePacket.Add("From", fromAddress);
			messagePacket.Add("Message", messageText);
			messagePacket.Add("MessageType", messageType);
			messagePacket.Add("MessageId", messageId);
			messagePacket.Add("SendTime", sendTime.Ticks.ToString());
			messagePacket.Add("ReceiveTime", receiveTime.Ticks.ToString());
			host.PostMessagePacket(messagePacket);			
		}
		private void PostDispatchMessage(string recordId, string toAddress, string fromAddress,
		                            		 string message, string messageType, string gateway,
		                            		 string backupGateway)
		{
			Hashtable messagePacket = new Hashtable();
			
			messagePacket.Add("PacketName", "DispatchMessage");
			messagePacket.Add("RecordId", recordId);
			messagePacket.Add("To", toAddress);
			messagePacket.Add("From", fromAddress);
			messagePacket.Add("Message", message);
			messagePacket.Add("MessageType", messageType);
			messagePacket.Add("Gateway", gateway);
			messagePacket.Add("BackupGateway", backupGateway);
			host.PostMessagePacket(messagePacket);			
		}
		private void PostSendResult(string recordId, string messageId, StatusCode status,
		                            string statusText, string errorCode, string errorText, bool sendStatusRequest)
		{
			Hashtable messagePacket = new Hashtable();
			
			messagePacket.Add("PacketName", "MessageOutResult");
			messagePacket.Add("RecordId", recordId);
			messagePacket.Add("MessageId", messageId);
			messagePacket.Add("StatusCode", ((int) status).ToString());
			messagePacket.Add("StatusText", statusText);
			messagePacket.Add("ErrorCode", errorCode);
			messagePacket.Add("ErrorText", errorText);
			if (sendStatusRequest) {
				messagePacket.Add("SendStatusRequest", "1");
			}
			else {
				messagePacket.Add("SendStatusRequest", "0");
			}
			host.PostMessagePacket(messagePacket);			
		}
		private void PostSendResultUpdate(string messageId, StatusCode status, string statusText, 
		                                  string errorCode, string errorText, DateTime receiveTime)
		{
			Hashtable messagePacket = new Hashtable();
			
			messagePacket.Add("PacketName", "MessageOutResultUpdate");
			messagePacket.Add("MessageId", messageId);
			messagePacket.Add("StatusCode", ((int) status).ToString());
			messagePacket.Add("StatusText", statusText);
			messagePacket.Add("ErrorCode", errorCode);
			messagePacket.Add("ErrorText", errorText);
			if (status == StatusCode.Received) messagePacket.Add("ReceiveTime", receiveTime.Ticks.ToString());
			host.PostMessagePacket(messagePacket);			
		}
		private void PostEventLog(string eventMessage, string eventDescription, EventLog eventType)
		{
			Hashtable messagePacket = new Hashtable();
			
			messagePacket.Add("PacketName", "EventLog");
			messagePacket.Add("EventType", ((int) eventType).ToString());
			messagePacket.Add("EventMessage", eventMessage);
			messagePacket.Add("EventDescription", eventDescription);
			host.PostMessagePacket(messagePacket);			
		}
		private void PostTraceLog(string traceLogMessage, TraceLog logType)
		{
			Hashtable messagePacket = new Hashtable();
			
			messagePacket.Add("PacketName", "TraceLog");
			messagePacket.Add("TraceLogType", ((int) logType).ToString());
			messagePacket.Add("TraceLogMessage", traceLogMessage);
			host.PostMessagePacket(messagePacket);			
		}
		private string GetPacketString(Hashtable hashtable, string key, string defaultValue)
		{
			try {
				return (string) hashtable[key];
			}
			catch {}
			return defaultValue;
		}

		private int GetPacketInteger(Hashtable hashtable, string key, int defaultValue)
		{
			try {
				return Convert.ToInt32((string) hashtable[key]);
			}
			catch {}
			return defaultValue;
		}

		private DateTime GetPacketDateTime(Hashtable hashtable, string key, DateTime defaultValue)
		{
			try {
				return new DateTime(Convert.ToInt64((string) hashtable[key]));
			}
			catch {}
			return defaultValue;
		}
		#endregion
	}
}
